package require SOAP


# -------------------------------------------------------------------------

# Description:
#   Procedure to generate the XML data for a configured SOAP procedure.
#   This is the XDAQ -wrapProc procedure
# Parameters:
#   procVarName - the path of the SOAP method configuration variable
#   args        - the arguments for this SOAP method
# Result:
#   XML data containing the SOAP method call.
# Notes:
#   We permit a small number of option to be specified on the method call
#   itself. -headers is used to set SOAP Header elements and -attr can be
#   used to set additional XML attributes on the method element (needed for
#   UDDI.)
#

proc ::SOAP::XDAQ_Make_Request {procVarName args}  {
   upvar $procVarName procvar
   insert "$procVarName"
   insert "$args"

    set procName [lindex [split $procVarName {_}] end]
    set params  $procvar(params)
    set name    $procvar(name)
    set uri     $procvar(uri)
    set soapenv $procvar(version)
    set soapenc $procvar(encoding)

    # Check for options (ie: -header) give up on the fist non-matching arg.
    array set opts {-headers {} -attributes {}}
    while {[string match -* [lindex $args 0]]} {
        switch -glob -- [lindex $args 0] {
            -header* {
                set opts(-headers) [concat $opts(-headers) [lindex $args 1]]
                set args [lreplace $args 0 0]
            }
            -attr* {
                set opts(-attributes) [concat $opts(-attributes) [lindex $args 1]]
                set args [lreplace $args 0 0]
            }
            -- {
                set args [lreplace $args 0 0]
                break
            }
            default {
                # stop option processing at the first invalid option.
                break
            }
        }
        set args [lreplace $args 0 0]
    }

    # check for variable number of params and set the num required.
    if {[lindex $params end] == "args"} {
        set n_params [expr ( [llength $params] - 1 ) / 2]
    } else {
        set n_params [expr [llength $params] / 2]
    }

    # check we have the correct number of parameters supplied.
    if {[llength $args] < $n_params} {
        set msg "wrong # args: should be \"$procName"
        foreach { id type } $params {
            append msg " " $id
        }
        append msg "\""
        error $msg
    }

    set doc [dom::DOMImplementation create]
    
    set envx [dom::document createElement $doc "SOAP-ENV:Envelope"]
    dom::element setAttribute $envx "xmlns:SOAP-ENV" $soapenv
    dom::element setAttribute $envx "SOAP-ENV:encodingStyle" $soapenc

    # Insert the empty Header
    set headelt [dom::document createElement $envx "SOAP-ENV:Header"]

    # Insert the body element and atributes.
    set bod [dom::document createElement $envx "SOAP-ENV:Body"]

    set cmd [dom::document createElement $bod "$name" ]

    # Insert any method attributes
    if {$opts(-attributes) != {}} {
        foreach {atname atvalue} $opts(-attributes) {
            dom::element setAttribute $cmd $atname $atvalue
        }
    }
    
    #    count the parameters (if any)
    set param_no 0
    foreach {key type} $params {incr param_no}
    
    if {$param_no > 0} {

        # Insert the parameter header
        set ParamH [dom::document createElement $cmd "Parameter"]
        dom::element setAttribute $ParamH name parameters
        dom::element setAttribute $ParamH type bag
    }

    # insert the parameters.
    set param_no 0
    foreach {key type} $params {
        set val [lindex $args $param_no]
        set Param [dom::document createElement $ParamH "Parameter"]
        dom::element setAttribute $Param name $key
        dom::element setAttribute $Param type $type
        dom::document createTextNode $Param $val
        incr param_no
    }

    # We have to strip out the DOCTYPE element though. It would be better to
    # remove the DOM node for this, but that didn't work.
    set prereq [dom::DOMImplementation serialize $doc]
    set req {}
    dom::DOMImplementation destroy $doc              ;# clean up
    regsub "<!DOCTYPE\[^>\]*>\r?\n?" $prereq {} req  ;# hack
    set req [encoding convertto utf-8 $req]          ;# make it UTF-8
    return $req                                      ;# return the XML data
}

proc ::SOAP::XDAQ_ExecConfigure {procVarName args}  {
   upvar $procVarName procvar
   insert "$procVarName"
   insert "$args"

    set procName [lindex [split $procVarName {_}] end]
    set params  $procvar(params)
    set name    $procvar(name)
    set uri     $procvar(uri)
    set soapenv $procvar(version)
    set soapenc $procvar(encoding)
   
   set xml "<?xml version=\"1.0\"?>"
   append xml "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"$soapenv\" SOAP-ENV:encodingStyle=\"$soapenc\">"
   append xml "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"$soapenv\">"
   append xml <SOAP-ENV:Header/>
   append xml <SOAP-ENV:Body>
   
   append xml "<Configure targetAddr=\"0\" id=\"[first $args]\">"
   
   set f [open [second $args] r]
   append xml [read $f]
   close $f
   
   append xml </Configure>
   
   append xml </SOAP-ENV:Body>
   append xml </SOAP-ENV:Envelope>
   
   set xml [regsub -all \n $xml ""]  ;#  remove all newlines
   set xml [regsub -all \r $xml ""]  ;#  remove all returns
   set xml [regsub -all \t $xml ""]  ;#  remove all tabs
   
   set xml [encoding convertto utf-8 $xml]          ;# make it UTF-8
   
   return $xml
}

proc ::SOAP::XDAQ_ExecAction {procVarName args}  {
   upvar $procVarName procvar
   insert "$procVarName"
   insert "$args"

    set procName [lindex [split $procVarName {_}] end]
    set params  $procvar(params)
    set name    $procvar(name)
    set uri     $procvar(uri)
    set soapenv $procvar(version)
    set soapenc $procvar(encoding)
   
   set xml "<?xml version=\"1.0\"?>"
   append xml "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"$soapenv\" SOAP-ENV:encodingStyle=\"$soapenc\">"
   append xml "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"$soapenv\">"
   append xml <SOAP-ENV:Header/>
   append xml <SOAP-ENV:Body>
   
   append xml "<$name targetAddr=\"0\" id=\"[first $args]\">"

   append xml </$name>
   
   append xml </SOAP-ENV:Body>
   append xml </SOAP-ENV:Envelope>
   
   set xml [regsub -all \n $xml ""]  ;#  remove all newlines
   set xml [regsub -all \r $xml ""]  ;#  remove all returns
   set xml [regsub -all \t $xml ""]  ;#  remove all tabs
   
   set xml [encoding convertto utf-8 $xml]          ;# make it UTF-8
   
   return $xml
}

SOAP::create XDAQ_Configure \
   -proxy http://localhost:8080 \
   -name Configure \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 20000  \
   -params {}
   
SOAP::create XDAQ_Enable \
   -proxy http://localhost:8080 \
   -name Enable \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}
   
SOAP::create XDAQ_Suspend \
   -proxy http://localhost:8080 \
   -name Suspend \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}
   
SOAP::create XDAQ_Resume \
   -proxy http://localhost:8080 \
   -name Resume \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}
   
SOAP::create XDAQ_Disable \
   -proxy http://localhost:8080 \
   -name Disable \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 10000  \
   -params {}
   
SOAP::create XDAQ_Halt \
   -proxy http://localhost:8080 \
   -name Halt \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}

SOAP::create XDAQ_Reset \
   -proxy http://localhost:8080 \
   -name Reset \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}

SOAP::create XDAQ_Query \
   -proxy http://localhost:8080 \
   -name ParameterQuery \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {}
   
SOAP::create XDAQ_WriteParameter \
   -proxy http://localhost:8080 \
   -name ParameterSet \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {XXX string}
   
SOAP::create XDAQ_ReadParameter \
   -proxy http://localhost:8080 \
   -name ParameterGet \
   -wrapProc ::SOAP::XDAQ_Make_Request \
   -timeout 5000  \
   -params {XXX string}
   
SOAP::create XDAQ_ExecConfigure \
   -proxy http://localhost:8080 \
   -name Configure  \
   -wrapProc ::SOAP::XDAQ_ExecConfigure \
   -timeout 10000  \
   -params {id int f string}
   
SOAP::create XDAQ_ExecHalt \
   -proxy http://localhost:8080 \
   -name Halt  \
   -wrapProc ::SOAP::XDAQ_ExecAction \
   -timeout 5000  \
   -params {id int}
   
SOAP::create XDAQ_ExecReset \
   -proxy http://localhost:8080 \
   -name Reset  \
   -wrapProc ::SOAP::XDAQ_ExecAction \
   -timeout 5000  \
   -params {id int}

SOAP::create XDAQ_ExecQuery \
   -proxy http://localhost:8080 \
   -name Query  \
   -wrapProc ::SOAP::XDAQ_ExecAction \
   -timeout 5000  \
   -params {id int}




